@@ -35,5 +35,3 @@
 #include "forms.h"
 #include "pages.h"
 #include "parser.h"
-#include "streamfuncs.h"
-#include "bootstrapper.h"
