@@ -3032,7 +3032,7 @@ pf_server_push (PFunArgs)
 
 	      called_yet++;
 
-	      output_fd = mhtml_stdout_fileno;
+	      output_fd = 1;
 
 	      if (nph)
 		bprintf (pushed, "HTTP/1.0 200\n");
