@@ -15,7 +15,7 @@
 **
 **  ======================================================================
 **
-**  Copyright (c) 1996,1997 Ralf S. Engelschall, All rights reserved.
+**  Copyright (c) 1996,1997,1998,1999 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This program is free software; it may be redistributed and/or modified
 **  only under the terms of either the Artistic License or the GNU General
@@ -135,7 +135,7 @@ void give_version(void)
 {
     fprintf(stdout, "%s\n", ePerl_Hello);
     fprintf(stdout, "\n");
-    fprintf(stdout, "Copyright (c) 1996-1997 Ralf S. Engelschall, All rights reserved.\n");
+    fprintf(stdout, "Copyright (c) 1996,1997,1998,1999 Ralf S. Engelschall <rse@engelschall.com>\n");
     fprintf(stdout, "\n");
     fprintf(stdout, "This program is distributed in the hope that it will be useful,\n");
     fprintf(stdout, "but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
@@ -351,6 +351,10 @@ int main(int argc, char **argv, char **env)
     char *cpHost;
     char *cpPort;
     char *cpPath;
+    char *cpCGIgi;
+    char *cpCGIpt;
+    char *cpCGIqs;
+    int fCGIqsEqualChar;
 
     /*  first step: our process initialisation */
     myinit();
@@ -468,25 +472,52 @@ int main(int argc, char **argv, char **env)
      *  determine source filename and runtime mode 
      */
 
-    if (optind == argc &&
-        getenv("GATEWAY_INTERFACE") != NULL &&
-        getenv("PATH_TRANSLATED")   != NULL   ) {
-        /*
-        **
-        **  Server-Side-Scripting-Language
-        **  (i.e. /url/to/nph-eperl/path/to/script.phtml)
-        **
-        */
+    if ((cpCGIgi = getenv("GATEWAY_INTERFACE")) == NULL)
+        cpCGIgi = "";
+    if ((cpCGIpt = getenv("PATH_TRANSLATED")) == NULL)
+        cpCGIpt = "";
+    if ((cpCGIqs = getenv("QUERY_STRING")) == NULL)
+        cpCGIqs = "";
+    fCGIqsEqualChar = FALSE;
+    if (cpCGIqs != NULL && strchr(cpCGIqs, '=') != NULL)
+        fCGIqsEqualChar = TRUE;
+
+    /*
+     *  Server-Side-Scripting-Language:
+     * 
+     *  Request:
+     *      /url/to/nph-eperl/url/to/script.phtml[?query-string]
+     *  Environment:
+     *      GATEWAY_INTERFACE=CGI/1.1
+     *      SCRIPT_NAME=/url/to/nph-eperl
+     *      SCRIPT_FILENAME=/path/to/nph-eperl
+     *      PATH_INFO=/url/to/script.phtml
+     *      PATH_TRANSLATED=/path/to/script.phtml
+     *      a) QUERY_STRING=""
+     *         optind=argc
+     *      b) QUERY_STRING=query-string (containing "=" char)
+     *         optind=argc 
+     *      c) QUERY_STRING=query-string (containing NO "=" char)
+     *         optind=argc-1
+     *         argv[optind]=query-string
+     */
+    if (   cpCGIgi[0] != NUL 
+        && cpCGIpt[0] != NUL
+        && (   (   optind == argc
+                && (   cpCGIqs[0] == NUL 
+                    || fCGIqsEqualChar      ) ) 
+            || (   optind == argc-1 
+                && !fCGIqsEqualChar 
+                && stringEQ(argv[optind], cpCGIqs) )      ) ) {
         
-        cp = getenv("GATEWAY_INTERFACE");
-        if (strncasecmp(cp, "CGI/1", 5) != 0) {
+        if (strncasecmp(cpCGIgi, "CGI/1", 5) != 0) {
             fprintf(stderr, "ePerl:Error: Unknown gateway interface: NOT CGI/1.x\n");
             CU(EX_IOERR);
         }
 
         /*  CGI/1.1 or NPH-CGI/1.1 script, 
             source in PATH_TRANSLATED. */
-        source = getenv("PATH_TRANSLATED");
+        source = cpCGIpt;
 
         /*  determine whether pure CGI or NPH-CGI mode */ 
         if ((cp = getenv("SCRIPT_FILENAME")) != NULL) { 
@@ -505,20 +536,41 @@ int main(int argc, char **argv, char **env)
         }
 
         /* set the command line for ``ps'' output */
-        sprintf(ca, "%s %s [%sCGI]", argv[0], source, mode == MODE_NPHCGI ? "NPH-" : "");
+        sprintf(ca, "%s %s [%sCGI/SSSL]", argv[0], source, mode == MODE_NPHCGI ? "NPH-" : "");
         argv[0] = strdup(ca);
     }
-    else if (optind == argc-1 &&
-             getenv("GATEWAY_INTERFACE") != NULL) {
-        /*
-        **
-        **  Stand-Alone inside Webserver environment
-        **  (i.e. CGI-script with shebang #!/path/to/eperl)
-        **
-        */
+    /*
+     *  Stand-Alone inside Webserver environment:
+     *
+     *  Request:
+     *      /url/to/script.cgi[/path-info][?query-string]
+     *      [script.cgi has shebang #!/path/to/eperl]
+     *  Environment:
+     *      GATEWAY_INTERFACE=CGI/1.1
+     *      SCRIPT_NAME=/url/to/script.cgi
+     *      SCRIPT_FILENAME=/path/to/script.cgi
+     *      PATH_INFO=/path-info
+     *      PATH_TRANSLATED=/path/to/docroot/path-info
+     *      a) QUERY_STRING=""
+     *         optind=argc-1
+     *         argv[optind]=/path/to/script.cgi
+     *      b) QUERY_STRING=query-string (containing "=" char)
+     *         optind=argc-1
+     *         argv[optind]=/path/to/script.cgi
+     *      c) QUERY_STRING=query-string (containing NO "=" char)
+     *         optind=argc-2
+     *         argv[optind]=/path/to/script.cgi
+     *         argv[optind+1]=query-string
+     */
+    else if (   cpCGIgi[0] != NUL
+             && ( (   optind == argc-1 
+                   && (   cpCGIqs[0] == NUL 
+                       || fCGIqsEqualChar      ) ) ||
+                  (   optind == argc-2 
+                   && !fCGIqsEqualChar 
+                   && stringEQ(argv[optind+1], cpCGIqs)) ) ) {
 
-        cp = getenv("GATEWAY_INTERFACE");
-        if (strncasecmp(cp, "CGI/1", 5) != 0) {
+        if (strncasecmp(cpCGIgi, "CGI/1", 5) != 0) {
             fprintf(stderr, "ePerl:Error: Unknown gateway interface: NOT CGI/1.x\n");
             CU(EX_IOERR);
         }
@@ -544,18 +596,28 @@ int main(int argc, char **argv, char **env)
         }
 
         /* set the command line for ``ps'' output */
-        sprintf(ca, "%s %s [%sCGI]", argv[0], source, mode == MODE_NPHCGI ? "NPH-" : "");
+        sprintf(ca, "%s %s [%sCGI/stand-alone]", argv[0], source, mode == MODE_NPHCGI ? "NPH-" : "");
         argv[0] = strdup(ca);
     }
-    else if (optind == argc-1 && 
-             getenv("GATEWAY_INTERFACE") == NULL &&
-             getenv("PATH_TRANSLATED")   == NULL &&
-             getenv("QUERY_STRING")      == NULL   ) {
-        /*
-        **
-        **  Stand-Alone outside Webserver environment
-        **
-        */
+    /*
+     *  Stand-Alone outside Webserver environment:
+     *
+     *  Request:
+     *      eperl script
+     *  Environment:
+     *      GATEWAY_INTERFACE=""
+     *      SCRIPT_NAME=""
+     *      SCRIPT_FILENAME=""
+     *      PATH_INFO=""
+     *      PATH_TRANSLATED=""
+     *      QUERY_STRING=""
+     *      optind=argc-1
+     *      argv[optind]=script
+     */
+    else if (   cpCGIgi[0] == NUL 
+             && cpCGIpt[0] == NUL
+             && cpCGIqs[0] == NUL 
+             && optind == argc-1  ) {
 
         /*  stand-alone filter, source as argument:
             either manually on the console or via shebang */
@@ -563,7 +625,7 @@ int main(int argc, char **argv, char **env)
         mode   = (mode == MODE_UNKNOWN ? MODE_FILTER : mode);
 
         /*  provide flexibility by recognizing "-" for stdin */
-        if (strcmp(source, "-") == 0) {
+        if (stringEQ(source, "-")) {
             /* store stdin to tmpfile */
             source = mytmpfile("ePerl.stdin");
             if ((fp = fopen(source, "w")) == NULL) {
@@ -579,8 +641,10 @@ int main(int argc, char **argv, char **env)
             keepcwd = TRUE;
         }
     }
+    /* 
+     *   Any other calling environment is an error...
+     */
     else {
-        /* else we are used in a wrong way... */
         fprintf(stderr, "ePerl:Error: Missing required file to process\n");
         fprintf(stderr, "ePerl:Error: Use either a filename, `-' for STDIN or PATH_TRANSLATED.\n");
         fprintf(stderr, "Try `%s --help' for more information.\n", progname);
@@ -607,11 +671,11 @@ int main(int argc, char **argv, char **env)
 
     /* the built-in GIF images */
     if ((mode == MODE_CGI || mode == MODE_NPHCGI) && (cp = getenv("PATH_INFO")) != NULL) { 
-        if (strcmp(cp, "/logo.gif") == 0) {
+        if (stringEQ(cp, "/logo.gif")) {
             give_img_logo();
             myexit(0);
         }
-        else if (strcmp(cp, "/powered.gif") == 0) {
+        else if (stringEQ(cp, "/powered.gif")) {
             give_img_powered();
             myexit(0);
         }
@@ -657,7 +721,7 @@ int main(int argc, char **argv, char **env)
             n = strlen(source);
             for (i = 0; allowed_file_ext[i] != NULL; i++) {
                 k = strlen(allowed_file_ext[i]);
-                if (strcmp(source+n-k, allowed_file_ext[i]) == 0) 
+                if (stringEQ(source+n-k, allowed_file_ext[i])) 
                     allow = TRUE;
             }
             if (!allow) {
@@ -713,7 +777,7 @@ int main(int argc, char **argv, char **env)
                             pw2 = getpwuid(atoi(allowed_caller_uid[i]));
                         else
                             pw2 = getpwnam(allowed_caller_uid[i]);
-                        if (strcmp(pw->pw_name, pw2->pw_name) == 0) {
+                        if (stringEQ(pw->pw_name, pw2->pw_name)) {
                             allow = TRUE;
                             break;
                         }
@@ -877,7 +941,7 @@ int main(int argc, char **argv, char **env)
         if ((cpHost = getenv("SERVER_NAME")) == NULL)
             cpHost = "localhost";
         cpPort = getenv("SERVER_PORT");
-        if (strcmp(cpPort, "80") == 0)
+        if (stringEQ(cpPort, "80"))
             cpPort = NULL;
         sprintf(ca, "http://%s%s%s%s", 
                 cpHost, cpPort != NULL ? ":" : "", cpPort != NULL ? cpPort : "", cpPath);
@@ -1130,8 +1194,8 @@ int main(int argc, char **argv, char **env)
     /* now when the request was not a HEAD request we create the output */
     cp = getenv("REQUEST_METHOD");
     if (! ((mode == MODE_CGI || mode == MODE_NPHCGI) &&
-           cp != NULL && strcmp(cp, "HEAD") == 0)) {
-        if (outputfile != NULL && strcmp(outputfile, "-") != 0) {
+           cp != NULL && stringEQ(cp, "HEAD"))) {
+        if (outputfile != NULL && stringNE(outputfile, "-")) {
             /* if we remembered current working dir, restore it now */
             if (mode == MODE_FILTER && cwd[0] != NUL)
                 chdir(cwd);
