@@ -287,7 +287,8 @@ static wordptr BITMASKTAB;
     ((*(address+(index>>LOGBITS)) AND BITMASKTAB[index AND MODMASK]) != 0)
 
 #define BIT_VECTOR_FLP_BIT(address,index,mask) \
-    (((*(addr+(index>>LOGBITS)) ^= (mask = BITMASKTAB[index AND MODMASK])) AND mask) != 0)
+    (mask = BITMASKTAB[index AND MODMASK]), \
+    (((*(addr+(index>>LOGBITS)) ^= mask) AND mask) != 0)
 
 #define BIT_VECTOR_DIGITIZE(type,value,digit) \
     value = (type) ((digit = value) / 10); \
@@ -502,7 +503,7 @@ N_word BitVector_Mask(N_int bits)           /* bit vector mask (unused bits) */
 
 charptr BitVector_Version(void)
 {
-    return((charptr)"5.1");
+    return((charptr)"5.6");
 }
 
 N_int BitVector_Word_Bits(void)
@@ -1466,6 +1467,7 @@ Z_int BitVector_Compare(wordptr X, wordptr Y)               /* X <,=,> Y ?   */
 
 charptr BitVector_to_Hex(wordptr addr)
 {
+    N_word  bits = bits_(addr);
     N_word  size = size_(addr);
     N_word  value;
     N_word  count;
@@ -1473,25 +1475,26 @@ charptr BitVector_to_Hex(wordptr addr)
     N_word  length;
     charptr string;
 
-    length = (size * (BITS >> 2)) + 1;
-    string = (charptr) malloc((size_t) length);
+    length = bits >> 2;
+    if (bits AND 0x0003) length++;
+    string = (charptr) malloc((size_t) (length+1));
     if (string == NULL) return(NULL);
     string += length;
-    *(--string) = (N_char) '\0';
+    *string = (N_char) '\0';
     if (size > 0)
     {
         *(addr+size-1) &= mask_(addr);
-        while (size-- > 0)
+        while ((size-- > 0) and (length > 0))
         {
             value = *addr++;
             count = BITS >> 2;
-            while (count-- > 0)
+            while ((count-- > 0) and (length > 0))
             {
                 digit = value AND 0x000F;
                 if (digit > 9) digit += (N_word) 'A' - 10;
                 else           digit += (N_word) '0';
-                *(--string) = (N_char) digit;
-                if (count > 0) value >>= 4;
+                *(--string) = (N_char) digit; length--;
+                if ((count > 0) and (length > 0)) value >>= 4;
             }
         }
     }
@@ -2488,7 +2491,7 @@ ErrCode BitVector_Mul_Pos(wordptr X, wordptr Y, wordptr Z)
 
 ErrCode BitVector_Multiply(wordptr X, wordptr Y, wordptr Z)
 {
-    ErrCode error;
+    ErrCode error = ErrCode_Ok;
     N_word  bit_x = bits_(X);
     N_word  bit_y = bits_(Y);
     N_word  bit_z = bits_(Z);
@@ -2627,7 +2630,7 @@ ErrCode BitVector_Div_Pos(wordptr Q, wordptr X, wordptr Y, wordptr R)
 
 ErrCode BitVector_Divide(wordptr Q, wordptr X, wordptr Y, wordptr R)
 {
-    ErrCode error;
+    ErrCode error = ErrCode_Ok;
     N_word  bits = bits_(Q);
     N_word  size = size_(Q);
     N_word  mask = mask_(Q);
@@ -3038,8 +3041,8 @@ N_int Set_Norm(wordptr addr)                                /* = | X |       */
         c = *addr++;
         while (c)
         {
-            if (c AND LSB) count++;
-            c >>= 1;
+            c &= c - 1;
+            count++;
         }
     }
     return(count);
@@ -3254,24 +3257,41 @@ void Matrix_Transpose(wordptr X, N_int rowsX, N_int colsX,
 }
 
 /*****************************************************************************/
-/*  VERSION:  5.0                                                            */
+/*  VERSION:  5.6                                                            */
 /*****************************************************************************/
 /*  VERSION HISTORY:                                                         */
 /*****************************************************************************/
 /*                                                                           */
-/*    01.03.98    Version 5.0  major additions, new functions, rewrites      */
-/*    16.07.97    Version 4.2  added is_empty, is_full                       */
-/*    30.06.97    Version 4.1  added word-ins/del, move-left/right, inc/dec  */
-/*    14.04.97    Version 4.0                                                */
-/*    04.02.97    Version 3.2                                                */
-/*    21.01.97    Version 3.1                                                */
-/*    12.01.97    Version 3.0                                                */
-/*    14.12.96    Version 2.0                                                */
-/*    08.01.96    Version 1.1                                                */
-/*    14.12.95    Version 1.0                                                */
-/*    29.11.95    First C version under UNIX (for Perl module)               */
-/*    01.11.93    First C version (MS C Compiler on PC with DOS)             */
-/*    ca. 1989    Created - Turbo Pascal version under CP/M on Apple ][+     */
+/*    Version 5.6  02.11.98  Leading zeros eliminated in "to_Hex()".         */
+/*    Version 5.5  21.09.98  Fixed bug of uninitialized "error" in Multiply. */
+/*    Version 5.4  07.09.98  Fixed bug of uninitialized "error" in Divide.   */
+/*    Version 5.3  12.05.98  Improved Norm. Completed history.               */
+/*    Version 5.2  31.03.98  Improved Norm.                                  */
+/*    Version 5.1  09.03.98  No changes.                                     */
+/*    Version 5.0  01.03.98  Major additions and rewrite.                    */
+/*    Version 4.2  16.07.97  Added is_empty, is_full.                        */
+/*    Version 4.1  30.06.97  Added word-ins/del, move-left/right, inc/dec.   */
+/*    Version 4.0  23.04.97  Rewrite. Added bit shift and bool. matrix ops.  */
+/*    Version 3.2  04.02.97  Added interval methods.                         */
+/*    Version 3.1  21.01.97  Fixed bug on 64 bit machines.                   */
+/*    Version 3.0  12.01.97  Added flip.                                     */
+/*    Version 2.0  14.12.96  Efficiency and consistency improvements.        */
+/*    Version 1.1  08.01.96  Added Resize and ExclusiveOr.                   */
+/*    Version 1.0  14.12.95  First version under UNIX (with Perl module).    */
+/*    Version 0.9  01.11.93  First version of C library under MS-DOS.        */
+/*    Version 0.1  ??.??.89  First version in Turbo Pascal under CP/M.       */
+/*                                                                           */
+/*****************************************************************************/
+/*  AUTHOR:                                                                  */
+/*****************************************************************************/
+/*                                                                           */
+/*    Steffen Beyer                                                          */
+/*    Ainmillerstr. 5 / App. 513                                             */
+/*    D-80801 Munich                                                         */
+/*    Germany                                                                */
+/*                                                                           */
+/*    mailto:sb@engelschall.com                                              */
+/*    http://www.engelschall.com/u/sb/download/                              */
 /*                                                                           */
 /*****************************************************************************/
 /*  COPYRIGHT:                                                               */
@@ -3284,13 +3304,22 @@ void Matrix_Transpose(wordptr X, N_int rowsX, N_int colsX,
 /*  LICENSE:                                                                 */
 /*****************************************************************************/
 /*                                                                           */
-/*    This piece of software is "Non-Profit-Ware" ("NP-ware").               */
+/*    This library is free software; you can redistribute it and/or          */
+/*    modify it under the terms of the GNU Library General Public            */
+/*    License as published by the Free Software Foundation; either           */
+/*    version 2 of the License, or (at your option) any later version.       */
 /*                                                                           */
-/*    You may use, copy, modify and redistribute it under                    */
-/*    the terms of the "Non-Profit-License" (NPL).                           */
+/*    This library is distributed in the hope that it will be useful,        */
+/*    but WITHOUT ANY WARRANTY; without even the implied warranty of         */
+/*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       */
+/*    Library General Public License for more details.                       */
 /*                                                                           */
-/*    Please refer to the file "NONPROFIT" in this distribution              */
-/*    for details!                                                           */
+/*    You should have received a copy of the GNU Library General Public      */
+/*    License along with this library; if not, write to the                  */
+/*    Free Software Foundation, Inc.,                                        */
+/*    59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                  */
+/*                                                                           */
+/*    or download a copy from ftp://ftp.gnu.org/pub/gnu/COPYING.LIB-2.0      */
 /*                                                                           */
 /*****************************************************************************/
 #endif
